<?php

namespace elanpl\L3;

class Config{
    public function getRequest(){
        return new Request();
    }

    public function getRouting(){
        return new \app\config\routing();
    }

    public function getBaseDirectory(){
        return _L3_BASE_PATH;
    }

    public function getApplicationDirectory(){
        return "app";
    }

    public function getControllersDirectory(){
        return "controllers";
    }

    public function getViewsDirectory(){
        return "views";
    }
}